@@ -8,9 +8,9 @@ import (
 	"strings"
 	"time"
 
-	"github.com/canonical/go-dqlite"
-	"github.com/canonical/go-dqlite/client"
-	"github.com/canonical/go-dqlite/internal/protocol"
+	"github.com/canonical/go-dqlite/v2"
+	"github.com/canonical/go-dqlite/v2/client"
+	"github.com/canonical/go-dqlite/v2/internal/protocol"
 )
 
 // Option can be used to tweak app parameters.
@@ -330,5 +330,5 @@ func defaultLogFunc(l client.LogLevel, format string, a ...interface{}) {
 		return
 	}
 	msg := fmt.Sprintf("["+l.String()+"]"+" dqlite: "+format, a...)
-	log.Printf(msg)
+	log.Printf("%s", msg)
 }
