@@ -83,7 +83,7 @@ import (
 	"time"
 	"unsafe"
 
-	"github.com/canonical/go-dqlite/internal/protocol"
+	"github.com/canonical/go-dqlite/v2/internal/protocol"
 )
 
 type Node struct {
@@ -232,7 +232,7 @@ func (s *Node) Close() {
 
 // Remark that Recover doesn't take the node role into account
 func (s *Node) Recover(cluster []protocol.NodeInfo) error {
-	for i, _ := range cluster {
+	for i := range cluster {
 		cluster[i].Role = protocol.Voter
 	}
 	return s.RecoverExt(cluster)
@@ -252,11 +252,22 @@ func (s *Node) RecoverExt(cluster []protocol.NodeInfo) error {
 		C.setInfo(infos, C.unsigned(i), cid, caddress, crole)
 	}
 	if rc := C.dqlite_node_recover_ext(server, infos, n); rc != 0 {
-		return fmt.Errorf("recover failed with error code %d", rc)
+		errmsg := C.GoString(C.dqlite_node_errmsg(server))
+		return fmt.Errorf("recover failed with error code %d, error details: %s", rc, errmsg)
 	}
 	return nil
 }
 
+func (s *Node) DescribeLastEntry() (uint64, uint64, error) {
+	server := (*C.dqlite_node)(unsafe.Pointer(s.node))
+	index := C.uint64_t(0)
+	term := C.uint64_t(0)
+	if rc := C.dqlite_node_describe_last_entry(server, &index, &term); rc != 0 {
+		return 0, 0, fmt.Errorf("dqlite_node_describe_last_entry failed with error code %d", rc)
+	}
+	return uint64(index), uint64(term), nil
+}
+
 // GenerateID generates a unique ID for a server.
 func GenerateID(address string) uint64 {
 	caddress := C.CString(address)
